@@ -20,17 +20,12 @@
 # ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-
-import sys
 import re
 import dns
-import os
 import json
 
-import bitcoin
-import dnssec
-from util import print_error
-from i18n import _
+from . import bitcoin
+from . import dnssec
 
 
 class Contacts(dict):
@@ -118,7 +113,7 @@ class Contacts(dict):
             return None
             
     def _validate(self, data):
-        for k,v in data.items():
+        for k,v in list(data.items()):
             if k == 'contacts':
                 return self._validate(v)
             if not bitcoin.is_address(k):
